@@ -50,12 +50,45 @@
 #include "query-local-address.hh"
 #include "tcpiohandler.hh"
 #include "ednsoptions.hh"
-
+#include "ednspadding.hh"
 #include "rec-protozero.hh"
 #include "uuid-utils.hh"
 #include "rec-tcpout.hh"
 
 thread_local TCPOutConnectionManager t_tcp_manager;
+std::shared_ptr<Logr::Logger> g_slogout;
+bool g_paddingOutgoing;
+
+void remoteLoggerQueueData(RemoteLoggerInterface& r, const std::string& data)
+{
+  auto ret = r.queueData(data);
+
+  switch (ret) {
+  case RemoteLoggerInterface::Result::Queued:
+    break;
+  case RemoteLoggerInterface::Result::PipeFull: {
+    const auto msg = RemoteLoggerInterface::toErrorString(ret);
+    const auto name = r.name();
+    SLOG(g_log << Logger::Debug << name << ": " << msg <<std::endl,
+         g_slog->withName(name)->info(Logr::Debug, msg));
+    break;
+  }
+  case RemoteLoggerInterface::Result::TooLarge: {
+    const auto msg = RemoteLoggerInterface::toErrorString(ret);
+    const auto name = r.name();
+    SLOG(g_log << Logger::Notice << name << ": " << msg <<endl,
+         g_slog->withName(name)->info(Logr::Debug, msg));
+    break;
+  }
+  case RemoteLoggerInterface::Result::OtherError: {
+    const auto msg = RemoteLoggerInterface::toErrorString(ret);
+    const auto name = r.name();
+    SLOG(g_log << Logger::Warning << name << ": " << msg << std::endl,
+         g_slog->withName(name)->info(Logr::Warning, msg));
+    break;
+  }
+  }
+}
 
 #ifdef HAVE_FSTRM
 #include "dnstap.hh"
@@ -87,7 +120,7 @@ static void logFstreamQuery(const std::shared_ptr<std::vector<std::unique_ptr<Fr
   DnstapMessage message(str, DnstapMessage::MessageType::resolver_query, SyncRes::s_serverID, &localip, &ip, protocol, reinterpret_cast<const char*>(&*packet.begin()), packet.size(), &ts, nullptr, auth);
 
   for (auto& logger : *fstreamLoggers) {
-    logger->queueData(str);
+    remoteLoggerQueueData(*logger, str);
   }
 }
 
@@ -116,7 +149,7 @@ static void logFstreamResponse(const std::shared_ptr<std::vector<std::unique_ptr
   DnstapMessage message(str, DnstapMessage::MessageType::resolver_response, SyncRes::s_serverID, &localip, &ip, protocol, reinterpret_cast<const char*>(packet.data()), packet.size(), &ts1, &ts2, auth);
 
   for (auto& logger : *fstreamLoggers) {
-    logger->queueData(str);
+    remoteLoggerQueueData(*logger, str);
   }
 }
 
@@ -174,7 +207,7 @@ static void logOutgoingQuery(const std::shared_ptr<std::vector<std::unique_ptr<R
 
   for (auto& logger : *outgoingLoggers) {
     if (logger->logQueries()) {
-      logger->queueData(buffer);
+      remoteLoggerQueueData(*logger, buffer);
     }
   }
 }
@@ -244,7 +277,7 @@ static void logIncomingResponse(const std::shared_ptr<std::vector<std::unique_pt
 
   for (auto& logger : *outgoingLoggers) {
     if (logger->logResponses()) {
-      logger->queueData(buffer);
+      remoteLoggerQueueData(*logger, buffer);
     }
   }
 }
@@ -261,6 +294,7 @@ static bool tcpconnect(const struct timeval& now, const ComboAddress& ip, TCPOut
   const struct timeval timeout{ g_networkTimeoutMsec / 1000, static_cast<suseconds_t>(g_networkTimeoutMsec) % 1000 * 1000};
   Socket s(ip.sin4.sin_family, SOCK_STREAM);
   s.setNonBlocking();
+  setTCPNoDelay(s.getHandle());
   ComboAddress localip = pdns::getQueryLocalAddress(ip.sin4.sin_family, 0);
   s.bind(localip);
 
@@ -272,7 +306,8 @@ static bool tcpconnect(const struct timeval& now, const ComboAddress& ip, TCPOut
     // tlsParams.d_caStore
     tlsCtx = getTLSContext(tlsParams);
     if (tlsCtx == nullptr) {
-      g_log << Logger::Error << "DoT to " << ip << " requested but not available" << endl;
+      SLOG(g_log << Logger::Error << "DoT to " << ip << " requested but not available" << endl,
+           g_slogout->info(Logr::Error, "DoT requested but not available", "server", Logging::Loggable(ip)));
       dnsOverTLS = false;
     }
   }
@@ -290,6 +325,7 @@ static LWResult::Result tcpsendrecv(const ComboAddress& ip, TCPOutConnectionMana
   uint16_t tlen = htons(vpacket.size());
   const char *lenP = reinterpret_cast<const char*>(&tlen);
 
+  len = 0; // in case of error
   localip.sin4.sin_family = ip.sin4.sin_family;
   if (getsockname(connection.d_handler->getDescriptor(), reinterpret_cast<sockaddr*>(&localip), &slen) != 0) {
     return LWResult::Result::PermanentError;
@@ -324,6 +360,25 @@ static LWResult::Result tcpsendrecv(const ComboAddress& ip, TCPOutConnectionMana
   return LWResult::Result::Success;
 }
 
+static void addPadding(const DNSPacketWriter& pw, size_t bufsize, DNSPacketWriter::optvect_t& opts)
+{
+  const size_t currentSize = pw.getSizeWithOpts(opts);
+  if (currentSize < (bufsize - 4)) {
+    const size_t remaining = bufsize - (currentSize + 4);
+    /* from rfc8647, "4.1.  Recommended Strategy: Block-Length Padding":
+       Clients SHOULD pad queries to the closest multiple of 128 octets.
+       Note we are in the client role here.
+    */
+    const size_t blockSize = 128;
+    const size_t modulo = (currentSize + 4) % blockSize;
+    size_t padSize = 0;
+    if (modulo > 0) {
+      padSize = std::min(blockSize - modulo, remaining);
+    }
+    opts.emplace_back(EDNSOptionCode::PADDING, makeEDNSPaddingOptString(padSize));
+  }
+}
+
 /** lwr is only filled out in case 1 was returned, and even when returning 1 for 'success', lwr might contain DNS errors
     Never throws! 
  */
@@ -337,6 +392,7 @@ static LWResult::Result asyncresolve(const ComboAddress& ip, const DNSName& doma
   //  string mapped0x20=dns0x20(domain);
   uint16_t qid = dns_random_uint16();
   DNSPacketWriter pw(vpacket, domain, type);
+  bool dnsOverTLS = SyncRes::s_dot_to_port_853 && ip.getPort() == 853;
 
   pw.getHeader()->rd=sendRDQuery;
   pw.getHeader()->id=qid;
@@ -368,7 +424,11 @@ static LWResult::Result asyncresolve(const ComboAddress& ip, const DNSName& doma
       weWantEDNSSubnet=true;
     }
 
-    pw.addOpt(g_outgoingEDNSBufsize, 0, g_dnssecmode == DNSSECMode::Off ? 0 : EDNSOpts::DNSSECOK, opts); 
+    if (dnsOverTLS && g_paddingOutgoing) {
+      addPadding(pw, bufsize, opts);
+    }
+
+    pw.addOpt(g_outgoingEDNSBufsize, 0, g_dnssecmode == DNSSECMode::Off ? 0 : EDNSOpts::DNSSECOK, opts);
     pw.commit();
   }
   lwr->d_rcode = 0;
@@ -381,7 +441,6 @@ static LWResult::Result asyncresolve(const ComboAddress& ip, const DNSName& doma
 
   boost::uuids::uuid uuid;
   const struct timeval queryTime = *now;
-  bool dnsOverTLS = SyncRes::s_dot_to_port_853 && ip.getPort() == 853;
 
   if (outgoingLoggers) {
     uuid = getUniqueID();
@@ -510,7 +569,11 @@ static LWResult::Result asyncresolve(const ComboAddress& ip, const DNSName& doma
 
     if(domain != mdp.d_qname) { 
       if(!mdp.d_qname.empty() && domain.toString().find((char)0) == string::npos /* ugly */) {// embedded nulls are too noisy, plus empty domains are too
-        g_log<<Logger::Notice<<"Packet purporting to come from remote server "<<ip.toString()<<" contained wrong answer: '" << domain << "' != '" << mdp.d_qname << "'" << endl;
+        SLOG(g_log<<Logger::Notice<<"Packet purporting to come from remote server "<<ip.toString()<<" contained wrong answer: '" << domain << "' != '" << mdp.d_qname << "'" << endl,
+             g_slogout->info(Logr::Notice, "Packet purporting to come from remote server contained wrong answer",
+                             "server", Logging::Loggable(ip),
+                             "qname", Logging::Loggable(domain),
+                             "onwire", Logging::Loggable(mdp.d_qname)));
       }
       // unexpected count has already been done @ pdns_recursor.cc
       goto out;
@@ -554,7 +617,9 @@ static LWResult::Result asyncresolve(const ComboAddress& ip, const DNSName& doma
   }
   catch (const std::exception &mde) {
     if (::arg().mustDo("log-common-errors")) {
-      g_log<<Logger::Notice<<"Unable to parse packet from remote server "<<ip.toString()<<": "<<mde.what()<<endl;
+      SLOG(g_log<<Logger::Notice<<"Unable to parse packet from remote server "<<ip.toString()<<": "<<mde.what()<<endl,
+           g_slogout->error(Logr::Notice, mde.what(), "Unable to parse packet from remote server", "server", Logging::Loggable(ip),
+                            "exception", Logging::Loggable("std::exception")));
     }
 
     lwr->d_rcode = RCode::FormErr;
@@ -568,11 +633,12 @@ static LWResult::Result asyncresolve(const ComboAddress& ip, const DNSName& doma
     return LWResult::Result::Success; // success - oddly enough
   }
   catch (...) {
-    g_log<<Logger::Notice<<"Unknown error parsing packet from remote server"<<endl;
+    SLOG(g_log<<Logger::Notice<<"Unknown error parsing packet from remote server"<<endl,
+         g_slogout->info(Logr::Notice, "Unknown error parsing packet from remote server", "server", Logging::Loggable(ip)));
   }
-  
-  g_stats.serverParseError++; 
-  
+
+  g_stats.serverParseError++;
+
  out:
   if (!lwr->d_rcode) {
     lwr->d_rcode=RCode::ServFail;
