@@ -500,7 +500,7 @@ static string doWipeCache(T begin, T end, uint16_t qtype)
   }
 
   int count = 0, pcount = 0, countNeg = 0;
-  for (auto wipe : toWipe) {
+  for (const auto& wipe : toWipe) {
     try {
       auto res = wipeCaches(wipe.first, wipe.second, qtype);
       count += res.record_count;
@@ -697,7 +697,7 @@ static string getNTAs()
 
   string ret("Configured Negative Trust Anchors:\n");
   auto luaconf = g_luaconfs.getLocal();
-  for (auto negAnchor : luaconf->negAnchors)
+  for (const auto& negAnchor : luaconf->negAnchors)
     ret += negAnchor.first.toLogString() + "\t" + negAnchor.second + "\n";
   return ret;
 }
@@ -803,9 +803,9 @@ static string getTAs()
 
   string ret("Configured Trust Anchors:\n");
   auto luaconf = g_luaconfs.getLocal();
-  for (auto anchor : luaconf->dsAnchors) {
+  for (const auto& anchor : luaconf->dsAnchors) {
     ret += anchor.first.toLogString() + "\n";
-    for (auto e : anchor.second) {
+    for (const auto& e : anchor.second) {
       ret += "\t\t" + e.getZoneRepresentation() + "\n";
     }
   }
@@ -928,6 +928,106 @@ static uint64_t doGetThreadCPUMsec(int n)
   return tt.times.at(n);
 }
 
+static ProxyMappingStats_t* pleaseGetProxyMappingStats()
+{
+  auto ret = new ProxyMappingStats_t;
+  if (t_proxyMapping) {
+    for (const auto& [key, entry] : *t_proxyMapping) {
+      ret->emplace(std::make_pair(key, ProxyMappingCounts{entry.stats.netmaskMatches, entry.stats.suffixMatches}));
+    }
+  }
+  return ret;
+}
+
+static RemoteLoggerStats_t* pleaseGetRemoteLoggerStats()
+{
+  auto ret = make_unique<RemoteLoggerStats_t>();
+
+  if (t_protobufServers.servers) {
+    for (const auto& server : *t_protobufServers.servers) {
+      ret->emplace(std::make_pair(server->address(), server->getStats()));
+    }
+  }
+  return ret.release();
+}
+
+static string doGetProxyMappingStats()
+{
+  ostringstream ret;
+  ret << "subnet\t\t\tmatches\tsuffixmatches" << endl;
+  auto proxyMappingStats = broadcastAccFunction<ProxyMappingStats_t>(pleaseGetProxyMappingStats);
+  for (const auto& [key, entry] : proxyMappingStats) {
+    ret << key.toString() << '\t' << entry.netmaskMatches << '\t' << entry.suffixMatches << endl;
+  }
+  return ret.str();
+}
+
+static RemoteLoggerStats_t* pleaseGetOutgoingRemoteLoggerStats()
+{
+  auto ret = make_unique<RemoteLoggerStats_t>();
+
+  if (t_outgoingProtobufServers.servers) {
+    for (const auto& server : *t_outgoingProtobufServers.servers) {
+      ret->emplace(std::make_pair(server->address(), server->getStats()));
+    }
+  }
+  return ret.release();
+}
+
+#ifdef HAVE_FSTRM
+static RemoteLoggerStats_t* pleaseGetFramestreamLoggerStats()
+{
+  auto ret = make_unique<RemoteLoggerStats_t>();
+
+  if (t_frameStreamServersInfo.servers) {
+    for (const auto& server : *t_frameStreamServersInfo.servers) {
+      ret->emplace(std::make_pair(server->address(), server->getStats()));
+    }
+  }
+  return ret.release();
+}
+
+static RemoteLoggerStats_t* pleaseGetNODFramestreamLoggerStats()
+{
+  auto ret = make_unique<RemoteLoggerStats_t>();
+
+  if (t_nodFrameStreamServersInfo.servers) {
+    for (const auto& server : *t_nodFrameStreamServersInfo.servers) {
+      ret->emplace(std::make_pair(server->address(), server->getStats()));
+    }
+  }
+  return ret.release();
+}
+#endif
+
+static void remoteLoggerStats(const string& type, const RemoteLoggerStats_t& stats, ostringstream& outpustStream)
+{
+  if (stats.empty()) {
+    return;
+  }
+  for (const auto& [key, entry] : stats) {
+    outpustStream << entry.d_queued << '\t' << entry.d_pipeFull << '\t' << entry.d_tooLarge << '\t' << entry.d_otherError << '\t' << key << '\t' << type << endl;
+  }
+}
+
+static string getRemoteLoggerStats()
+{
+  ostringstream outputStream;
+  outputStream << "Queued\tPipe-\tToo-\tOther-\tAddress\tType" << endl;
+  outputStream << "\tFull\tLarge\terror" << endl;
+  auto stats = broadcastAccFunction<RemoteLoggerStats_t>(pleaseGetRemoteLoggerStats);
+  remoteLoggerStats("protobuf", stats, outputStream);
+  stats = broadcastAccFunction<RemoteLoggerStats_t>(pleaseGetOutgoingRemoteLoggerStats);
+  remoteLoggerStats("outgoingProtobuf", stats, outputStream);
+#ifdef HAVE_FSTRM
+  stats = broadcastAccFunction<RemoteLoggerStats_t>(pleaseGetFramestreamLoggerStats);
+  remoteLoggerStats("dnstapFrameStream", stats, outputStream);
+  stats = broadcastAccFunction<RemoteLoggerStats_t>(pleaseGetNODFramestreamLoggerStats);
+  remoteLoggerStats("dnstapNODFrameStream", stats, outputStream);
+#endif
+  return outputStream.str();
+}
+
 static uint64_t calculateUptime()
 {
   return time(nullptr) - g_stats.startupTime;
@@ -966,26 +1066,11 @@ static string doCurrentQueries()
   return broadcastAccFunction<string>(pleaseGetCurrentQueries);
 }
 
-uint64_t* pleaseGetThrottleSize()
-{
-  return new uint64_t(SyncRes::getThrottledServersSize());
-}
-
-static uint64_t getThrottleSize()
-{
-  return broadcastAccFunction<uint64_t>(pleaseGetThrottleSize);
-}
-
 static uint64_t getNegCacheSize()
 {
   return g_negCache->size();
 }
 
-uint64_t* pleaseGetEDNSStatusesSize()
-{
-  return new uint64_t(SyncRes::getEDNSStatusesSize());
-}
-
 uint64_t* pleaseGetConcurrentQueries()
 {
   return new uint64_t(getMT() ? getMT()->numProcesses() : 0);
@@ -1114,6 +1199,21 @@ static StatsMap toStatsMap(const string& name, const pdns::AtomicHistogram& hist
   return entries;
 }
 
+static StatsMap toAuthRCodeStatsMap(const string& name, const std::array<pdns::stat_t, 16>& v)
+{
+  const string pbasename = getPrometheusName(name);
+  StatsMap entries;
+
+  uint8_t n = 0;
+  for (const auto& entry : v) {
+    const auto key = RCode::to_short_s(n);
+    std::string pname = pbasename + "{rcode=\"" + key + "\"}";
+    entries.emplace("auth-" + key + "-answers", StatsMapEntry{pname, std::to_string(entry)});
+    n++;
+  }
+  return entries;
+}
+
 static StatsMap toCPUStatsMap(const string& name)
 {
   const string pbasename = getPrometheusName(name);
@@ -1153,6 +1253,64 @@ static StatsMap toRPZStatsMap(const string& name, LockGuarded<std::unordered_map
   return entries;
 }
 
+static StatsMap toProxyMappingStatsMap(const string& name)
+{
+  const string pbasename = getPrometheusName(name);
+  StatsMap entries;
+
+  auto proxyMappingStats = broadcastAccFunction<ProxyMappingStats_t>(pleaseGetProxyMappingStats);
+  size_t count = 0;
+  for (const auto& [key, entry] : proxyMappingStats) {
+    auto keyname = pbasename + "{netmask=\"" + key.toString() + "\",count=\"";
+    auto sname1 = name + "-n-" + std::to_string(count);
+    auto pname1 = keyname + "netmaskmatches\"}";
+    entries.emplace(sname1, StatsMapEntry{pname1, std::to_string(entry.netmaskMatches)});
+    auto sname2 = name + "-s-" + std::to_string(count);
+    auto pname2 = keyname + "suffixmatches\"}";
+    entries.emplace(sname2, StatsMapEntry{pname2, std::to_string(entry.suffixMatches)});
+    count++;
+  }
+  return entries;
+}
+
+static StatsMap toRemoteLoggerStatsMap(const string& name)
+{
+  const auto pbasename = getPrometheusName(name);
+  StatsMap entries;
+
+  std::vector<std::pair<RemoteLoggerStats_t, std::string>> list;
+  auto stats1 = broadcastAccFunction<RemoteLoggerStats_t>(pleaseGetRemoteLoggerStats);
+  list.emplace_back(stats1, "protobuf");
+  auto stats2 = broadcastAccFunction<RemoteLoggerStats_t>(pleaseGetOutgoingRemoteLoggerStats);
+  list.emplace_back(stats2, "outgoingProtobuf");
+#ifdef HAVE_FSTRM
+  auto stats3 = broadcastAccFunction<RemoteLoggerStats_t>(pleaseGetFramestreamLoggerStats);
+  list.emplace_back(stats3, "dnstapFrameStream");
+  auto stats4 = broadcastAccFunction<RemoteLoggerStats_t>(pleaseGetNODFramestreamLoggerStats);
+  list.emplace_back(stats4, "dnstapNODFrameStream");
+#endif
+  uint64_t count = 0;
+  for (const auto& [stats, type] : list) {
+    for (const auto& [key, entry] : stats) {
+      auto keyname = pbasename + "{address=\"" + key + "\",type=\"" + type + "\",count=\"";
+      auto sname1 = name + "-q-" + std::to_string(count);
+      auto pname1 = keyname + "queued\"}";
+      entries.emplace(sname1, StatsMapEntry{pname1, std::to_string(entry.d_queued)});
+      auto sname2 = name + "-p-" + std::to_string(count);
+      auto pname2 = keyname + "pipeFull\"}";
+      entries.emplace(sname2, StatsMapEntry{pname2, std::to_string(entry.d_pipeFull)});
+      auto sname3 = name + "-t-" + std::to_string(count);
+      auto pname3 = keyname + "tooLarge\"}";
+      entries.emplace(sname3, StatsMapEntry{pname3, std::to_string(entry.d_tooLarge)});
+      auto sname4 = name + "-o-" + std::to_string(count);
+      auto pname4 = keyname + "otherError\"}";
+      entries.emplace(sname4, StatsMapEntry{pname4, std::to_string(entry.d_otherError)});
+      ++count;
+    }
+  }
+  return entries;
+}
+
 static void registerAllStats1()
 {
   addGetStat("questions", &g_stats.qcounter);
@@ -1240,7 +1398,7 @@ static void registerAllStats1()
   addGetStat("max-mthread-stack", &g_stats.maxMThreadStackUsage);
 
   addGetStat("negcache-entries", getNegCacheSize);
-  addGetStat("throttle-entries", getThrottleSize);
+  addGetStat("throttle-entries", SyncRes::getThrottledServersSize);
 
   addGetStat("nsspeeds-entries", SyncRes::getNSSpeedsSize);
   addGetStat("failed-host-entries", SyncRes::getFailedServersSize);
@@ -1398,6 +1556,9 @@ static void registerAllStats1()
 
   addGetStat("idle-tcpout-connections", getCurrentIdleTCPConnections);
 
+  addGetStat("maintenance-usec", &g_stats.maintenanceUsec);
+  addGetStat("maintenance-calls", &g_stats.maintenanceCalls);
+
   /* make sure that the ECS stats are properly initialized */
   SyncRes::clearECSStats();
   for (size_t idx = 0; idx < SyncRes::s_ecsResponsesBySubnetSize4.size(); idx++) {
@@ -1418,6 +1579,15 @@ static void registerAllStats1()
   addGetStat("policy-hits", []() {
     return toRPZStatsMap("policy-hits", g_stats.policyHits);
   });
+  addGetStat("proxy-mapping-total", []() {
+    return toProxyMappingStatsMap("proxy-mapping-total");
+  });
+  addGetStat("auth-rcode-answers", []() {
+    return toAuthRCodeStatsMap("auth-rcode-answers", g_stats.authRCode);
+  });
+  addGetStat("remote-logger-count", []() {
+    return toRemoteLoggerStatsMap("remote-logger-count");
+  });
 }
 
 void registerAllStats()
@@ -1876,9 +2046,25 @@ static string setEventTracing(T begin, T end)
   }
 }
 
-static void* pleaseSupplantProxyMapping(std::shared_ptr<ProxyMapping> pm)
+static void* pleaseSupplantProxyMapping(const ProxyMapping& pm)
 {
-  t_proxyMapping = pm;
+  if (pm.empty()) {
+    t_proxyMapping = nullptr;
+  }
+  else {
+    // Copy the existing stats values, for the new config items also present in the old
+    auto newmapping = make_unique<ProxyMapping>();
+    for (const auto& [nm, entry] : pm) {
+      auto& newentry = newmapping->insert(nm);
+      newentry.second = entry;
+      if (t_proxyMapping) {
+        if (const auto* existing = t_proxyMapping->lookup(nm); existing != nullptr) {
+          newentry.second.stats = existing->second.stats;
+        }
+      }
+    }
+    t_proxyMapping = std::move(newmapping);
+  }
   return nullptr;
 }
 
@@ -1925,8 +2111,10 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
             "get-ntas                         get all configured Negative Trust Anchors\n"
             "get-tas                          get all configured Trust Anchors\n"
             "get-parameter [key1] [key2] ..   get configuration parameters\n"
+            "get-proxymapping-stats           get proxy mapping statistics\n"
             "get-qtypelist                    get QType statistics\n"
             "                                 notice: queries from cache aren't being counted yet\n"
+            "get-remotelogger-stats           get remote logger statistics\n"
             "hash-password [work-factor]      ask for a password then return the hashed version\n"
             "help                             get this list\n"
             "ping                             check that all threads are alive\n"
@@ -1987,7 +2175,7 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
     return doDumpToFile(s, pleaseDumpDoTProbeMap, cmd, false);
   }
   if (cmd == "dump-ednsstatus" || cmd == "dump-edns") {
-    return doDumpToFile(s, pleaseDumpEDNSMap, cmd);
+    return doDumpToFile(s, pleaseDumpEDNSMap, cmd, false);
   }
   if (cmd == "dump-nsspeeds") {
     return doDumpToFile(s, pleaseDumpNSSpeeds, cmd, false);
@@ -2002,7 +2190,7 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
     return doDumpRPZ(s, begin, end);
   }
   if (cmd == "dump-throttlemap") {
-    return doDumpToFile(s, pleaseDumpThrottleMap, cmd);
+    return doDumpToFile(s, pleaseDumpThrottleMap, cmd, false);
   }
   if (cmd == "dump-non-resolving") {
     return doDumpToFile(s, pleaseDumpNonResolvingNS, cmd, false);
@@ -2033,8 +2221,7 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
       ProxyMapping proxyMapping;
       loadRecursorLuaConfig(::arg()["lua-config-file"], delayedLuaThreads, proxyMapping);
       startLuaConfigDelayedThreads(delayedLuaThreads, g_luaconfs.getCopy().generation);
-      std::shared_ptr<ProxyMapping> ptr = proxyMapping.empty() ? nullptr : std::make_shared<ProxyMapping>(proxyMapping);
-      broadcastFunction([=] { return pleaseSupplantProxyMapping(ptr); });
+      broadcastFunction([=] { return pleaseSupplantProxyMapping(proxyMapping); });
       g_log << Logger::Warning << "Reloaded Lua configuration file '" << ::arg()["lua-config-file"] << "', requested via control channel" << endl;
       return {0, "Reloaded Lua configuration file '" + ::arg()["lua-config-file"] + "'\n"};
     }
@@ -2178,6 +2365,12 @@ RecursorControlChannel::Answer RecursorControlParser::getAnswer(int s, const str
   if (cmd == "set-event-trace-enabled") {
     return {0, setEventTracing(begin, end)};
   }
+  if (cmd == "get-proxymapping-stats") {
+    return {0, doGetProxyMappingStats()};
+  }
+  if (cmd == "get-remotelogger-stats") {
+    return {0, getRemoteLoggerStats()};
+  }
 
   return {1, "Unknown command '" + cmd + "', try 'help'\n"};
 }
